#!/usr/bin/env python
#
# Copyright (c) 2022, Alkemy Spa
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from dbt.adapters.base import AdapterPlugin
from dbt.adapters.sas.column import SasColumn
from dbt.adapters.sas.connections import SasConnectionManager
from dbt.adapters.sas.credentials import SasCredentials
from dbt.adapters.sas.impl import SasAdapter
from dbt.adapters.sas.relation import SasRelation
from dbt.include import sas

__all__ = ["SasConnectionManager", "SasCredentials", "SasAdapter", "SasRelation", "SasColumn", "Plugin", "sas"]

Plugin = AdapterPlugin(adapter=SasAdapter, credentials=SasCredentials, include_path=sas.PACKAGE_PATH)
